#!/usr/bin/perl -w

# evo2html 0.4
# A simple script to get stuff from your evolution task and calander files and put
# them in some html form.
#
#     Copyright (C) 2004, 2005,  Iain Buchanan
# 
#     This program is free software; you can redistribute it and/or modify
#     it under the terms of the GNU General Public License as published by
#     the Free Software Foundation; either version 2 of the License, or
#     (at your option) any later version.
#
#     This program is distributed in the hope that it will be useful,
#     but WITHOUT ANY WARRANTY; without even the implied warranty of
#     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#     GNU General Public License for more details.
#
#     You should have received a copy of the GNU General Public License
#     along with this program; if not, write to the Free Software
#     Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

use strict;
use Time::Local;
use POSIX qw(strftime);

my $home = './';
if (defined $ENV{'HOME'}) {
   $home = $ENV{'HOME'};
}

my $taskfile = '~/.evolution/tasks/local/system/tasks.ics';
my $taskhtml = '~/public_html/evotask.html';
my $template = '~/.evo2html.html';
my $complete = 0;
my $maxlines = 0;
my @todo  = ();
my $sum_date_fmt = '%d-%m-%Y ';
my $long_date_fmt = '%a, %b %e %Y';

#
# parse ARGV
#
while ($#ARGV + 1 > 0) {
   if    ($ARGV[0] =~ /^--task$/) { # location of task file
      shift @ARGV;
      $taskfile = shift @ARGV;
   }
   elsif ($ARGV[0] =~ /^--thtml$/) { # html output from parsing task file
      shift @ARGV;
      $taskhtml = shift @ARGV;
   }
   elsif ($ARGV[0] =~ /^--template$/) { # html template file other than ~/evo2html.conf
      shift @ARGV;
      $template = shift @ARGV;
   }
   elsif ($ARGV[0] =~ /^--home$/) { # override home
      shift @ARGV;
      $home = shift @ARGV;
   }
   elsif ($ARGV[0] =~ /^--complete$/) { # show completed tasks
      shift @ARGV;
      $complete = 1;
   }
   elsif ($ARGV[0] =~ /^--maxlines$/) { # show completed tasks
      shift @ARGV;
      $maxlines = shift @ARGV;
      die unless $maxlines =~ /^\d+$/;
   }
   elsif ($ARGV[0] =~ /(^--usage$)|(^--help$)|(^-h$)/) {
      shift @ARGV;
      print << "USAGE";

$0:
   will try and parse the evolution ics file into some nice html!

   evo2html version 0.4, Copyright (C) 2004, 2005, Iain Buchanan
   evo2html comes with ABSOLUTELY NO WARRANTY;  This is free software, and you
   are welcome to redistribute it under certain conditions; See the file README
   that came with this program for more details.

Override the defaults with:

--task <file>: location of ics file to parse (defaults to
   ~/.evolution/tasks/local/system/tasks.ics), but you could also use
   ~/.evolution/calendar/local/system/calendar.ics - for evolution 2.2.1.1 and
   similar, you might want to use something like
   ~/.evolution/tasks/local/1115346798.11405.4@<hostname>/tasks.ics

--thtml <file>: location of html output (defaults to
    ~/public_html/evotask.html).

--template <file>: location of html template file (defaults to
   ~/.evo2html.html)  This file must be found.

--home <dir>: '~/' in the above locations will be replaced with \$HOME,
    unless you override \$HOME here.  If \$HOME can not be determined, ./
    will be used instead.

--maxlines <num>: if num is a positive whole number, only prints first num/2 and
    last num/2 lines of any section longer than num.
USAGE
   exit;
   }
   else {
      die "unknown arguement: '$ARGV[0]'\n";
   }
}

$taskfile =~ s/^~\//$home\//;
$taskhtml =~ s/^~\//$home\//;
$template =~ s/^~\//$home\//;

&parse_ics_files (\@todo);
&print_html(\@todo);

sub parse_ics_files {
   my %elem;
   my $vtodo = 0;
   my $last;

   open (TASKS, "$taskfile") or die "couldn't open file '$taskfile': $!.\n" .
       " Try specifing a file with --task <filename>\n";
   while (<TASKS>) {
      chomp;

      if (/^BEGIN:VTODO$/) {
         $vtodo = 1;
         %elem = ();
         #print '<';
      }
      elsif (/^END:VTODO$/) {
         $vtodo = 0;
         push @{$_[0]},{%elem};
         # print ">\n";
      }
      elsif ($vtodo) {
         if (/^ /) {
            $elem{"$last"} = $elem{"$last"} . $_;
         }
         elsif (/^(.*?)(\;.*?)?:(.*)$/) {
            my $key = $1;
            my $val = $3;
            # &mangle (\$key, \$val);
            $elem{"$key"} = "$val";
            $last = $1;
         }
      }
   }
   close TASKS;
}

sub print_html {
   my @summary   = (0, 0);
   my @middle    = (0, 0);
   my @loop      = (0, 0);
   my $grey  = '#e8e8e8';
   my $white = '#ffffff';
   my $bc = $grey;

   open (TEMPL, "$template") or die  "couldn't open template '$template': $!\n" .
       " Try specifing a file with --template <filename>\n";
   my @lines = <TEMPL>;
   close TEMPL;

   #
   # find the loops in the template
   #
   for (my $i = 0; $i < $#lines; $i ++) {
      if ($lines[$i] =~ /<!-- begin summary loop -->/) {
         $summary[0] = $i;
      }
      elsif ($lines[$i] =~ /<!-- end summary loop -->/) {
         $summary[1] = $i - 1;
         $middle[0] = $i;
      }
      elsif ($lines[$i] =~ /<!-- begin long loop -->/) {
         $middle[1] = $i - 1;
         $loop[0] = $i;
      }
      elsif ($lines[$i] =~ /<!-- end long loop -->/) {
         $loop[1] = $i - 1;
      }
   }

   #
   # now spit 'em out!
   #
   open (THTML, ">$taskhtml") or die "couldn't open html output '$taskhtml': $!.\n" .
       " Try specifing a file with --thtml <filename>\n";
   my $output;
   # beginning
   foreach (0 .. $summary[0] - 1) {
      &output (*THTML{IO}, $lines[$_]);
   }
   # summary
   my $hidden = 0;
   foreach my $entry (@{$_[0]}) {
      if (!$complete && exists $entry->{'STATUS'} && $entry->{'STATUS'} eq 'COMPLETED') {
         $hidden ++;
      }
      else {
         $entry->{'ALT_COL'} = "$bc";
         if ($bc eq $white) { $bc = $grey;} else { $bc = $white; }
         foreach ($summary[0] .. $summary[1]) {
            $output = $lines[$_];
            $output =~ s/vtodo\.(.*?);/&strsp($entry->{"$1"}, "$sum_date_fmt")/ge;
            &output (*THTML{IO}, $output);
            
         }
      }
   }
   # middle
   foreach ($middle[0] .. $middle[1]) {
      &output (*THTML{IO}, $lines[$_]);
   }
   # loop
   $bc = $grey;
   foreach my $entry (@{$_[0]}) {
      if (!$complete && exists $entry->{'STATUS'} && $entry->{'STATUS'} eq 'COMPLETED') {
         $hidden ++;
      }
      else {
         $entry->{'ALT_COL'} = "$bc";
         if ($bc eq $white) { $bc = $grey;} else { $bc = $white; }
         foreach ($loop[0] .. $loop[1]) {
            $output = $lines[$_];
            $output =~ s/vtodo\.(.*?);/&strsp($entry->{"$1"}, "$long_date_fmt")/ge;
            &output (*THTML{IO}, $output);
         }
      }
   }
   # end
   foreach ($loop[1] + 1 .. $#lines) {
      &output (*THTML{IO}, $lines[$_]);
   }

   # free up array
   @lines = ();
   close THTML;
}

sub output {
   my $fh = shift;
   my $st = shift;
   print $fh $st, unless $st =~ /^<!--/;
}

sub strsp {
   my $check = shift;

   if (defined $check) {
      $check =~ s/(\\.)/&unbackslash($1)/eg;
      # &timelocal($6, $5, $4, $3, $2-1, $1)
      if ($check =~ s/^(\d{4})(\d{2})(\d{2})(T(\d{2})(\d{2})(\d{2}))?$/&nicetime("$_[0]", $7, $6, $5, $3, $2, $1)/eg) {
         ;
      }

      if ($maxlines > 0) {
         my @chlines = split /\n/,$check;
         if ($#chlines > $maxlines) {
            $check = '';
            foreach (0 .. $maxlines / 2) {
               $check .= $chlines[$_];
            }
            $check .= "\n<br><br><I>... some lines have been deleted ...<I><br><br>\n";
            foreach ((0 - $maxlines)/2 .. -1) {
               $check .= $chlines[$_];
            }
         }
      }

      return $check;
   }
   else {
      return '&nbsp';
   }
}

sub unbackslash {
   my $from = $_[0];
   if    ($from eq '\n') { return "\n<br>"; }
   elsif ($from eq '\r') { return "\n<br>"; }
   elsif ($from eq '\t') { return "\t"; }
   else { return chop $from; }
}

sub nicetime {
   if (!defined $_[1] || !defined $_[2] || !defined $_[3]) {
      return &strftime ("$_[0]", 0, 0, 0, $_[4], $_[5]-1, $_[6]-1900);
   }
   else {
      return &strftime ("$_[0]", $_[1], $_[2], $_[3], $_[4], $_[5]-1, $_[6]-1900);
   }
}
